/************************************************
**** BOT                                     ****
**** Operazioni di base per i robot.         ****
**** Tutti e 3 gli algoritmi derivano da     ****
**** questa classe.                          ****
****                                         ****
**** Alessandro Santuari   6 SI              ****
************************************************/

#include <stdio.h>
#include "BOT.h"


BOT::BOT(RInterface *ri) // COSTRUTTORE
{
	// la nostra interfaccia per l'ambiente esterno (es. sensori...)
	RI=ri;
	// la nostra memoria
	MyMap=new Grafo();
	// la funzione di isomorfismo che ci permettera di dialogare con l'interfaccia
	for (int i=0;i<MAX_VERTICES;i++)
	{
		Iso[i]=IsoInversa[i]=0;
	}

	// aggiungere alla mia mappa il vertice di partenza.
	int MyPosRI=RI->getRobotPos();
	int MyPosMAP=MyMap->AddVertice();
	InsertIso(MyPosMAP,MyPosRI);
}

BOT::~BOT()	// DISTRUTTORE
{
	delete MyMap;
}

void BOT::InsertIso(int from, int to)
{
	// inserisce un nuovo vertice nella funzione isomorfismo
   // from e' l'etichetta utilizzata dal robot (in base all'ordine di visita)
   // to   e' l'etichetta utilizzata dall'interfaccia (basata sulle etichette del grafo ambiente)
	Iso[from]=to;
	IsoInversa[to]=from;
}

void BOT::JustMoved(int OldPos)
{/*
   Il robot si e' appena mosso. Controlla la sua nuova posizione e capisce se e'
   arrivato in un vertice sconosciuto oppure no
*/
	int NewPos=getMyPos();			// la nostra nuova posizione
	if (NewPos==OldPos) return;	// non ci siamo mossi!
	if (NewPos!=0)
	{
		// Sono arrivato in un vertice gia' conosciuto!
		// pero' ho imparato un nuovo arco!
		MyMap->AddArco(OldPos,NewPos);
	}
	else
	{
		// sono arrivato in un nuovo vertice..
		int MyPosRI=RI->getRobotPos();	// come viene etichettato dall'interfaccia?
		int MyPosMAP=MyMap->AddVertice();// creo in memoria il nuovo vertice
		InsertIso(MyPosMAP,MyPosRI);		// nuova coppia per l'isomorfismo
		MyMap->AddArco(OldPos,MyPosMAP);	// scoperto un nuovo arco!
	}
}

int BOT::getMyPos()
{
	// restituisce il vertice sul quale si trova il robot secondo il sistema
	// di etichettamento del robot
	return RI_to_MyMap(RI->getRobotPos());
}

int BOT::RI_to_MyMap(int RI_v)
{/*
	Utilizza la funzione di isomorfismo per trasformare un etichetta
   espressa secondo il sistema dell'interfaccia ad un etichetta utilizzabile
   dal robot.
*/
	return IsoInversa[RI_v];
}

int BOT::MyMap_to_RI(int Map_v)
{/*
	Utilizza la funzione di isomorfismo per trasformare un etichetta
   espressa secondo il sistema di riferimento del robot ad un etichetta 
   utilizzabile dall'interfaccia.
*/
	return Iso[Map_v];
}

void BOT::Explore()
{/*
   Esplora tutto il grafo!
*/
   // chiama la funzione sovrascritta dai robot
	_explore();
   RI->Finish();
	printf("Mosse:%d Penalty:%d\n",RI->getMoves(),RI->getMoves()-MyMap->getNumArchi());
}

void BOT::SaveMap(char *fname)
{/*
   Salva la mappa prodotta dal robot applicando la funzione isoformismo calcolata 
   durante l'esplorazione per ottenere un file che dovrebbe essere uguale a quello del
   grafo-ambiente
*/
	MyMap->SaveIso(fname,Iso);
}

zTList<int> *BOT::FindPathTo(int u, int *color)
{/*
   Trovare un cammino minimo che congiunge il robot a u.
   Restituisce una lista di vertici che costituisce il cammino.
   Se color e' NULL viene ignorato, altrimenti vengono utilizzati
   solo i nodi di colore BLU (per l'algoritmo panaite)
*/
	int r=getMyPos();
	int dim=MyMap->getNumVertici()+1;
	int *T=(int*)malloc(sizeof(int)*dim);  //vettore dei padri per l'albero dei cammini
	int *D=(int*)malloc(sizeof(int)*dim);
	bool *B=(bool*)malloc(sizeof(bool)*dim);
   
	zTList<int> *Insieme=new zTList<int>(true);

   // inizializzazioni
	for (int i=0;i<dim;i++)
	{
		T[i]=r;
      D[i]=999999999;
		B[i]=false;
	}
	T[r]=0;D[r]=0;

	int *a=new int;
	*a=r;
	Insieme->Insert(a);  // inserisci il nodo di partenza
	
	while (!Insieme->IsEmpty())
	{
		int i;
		a=Insieme->First()->Read();
		i=*a;
		Insieme->First()->Remove();
		B[i]=false;
		zTList<int> *Adia=MyMap->getAdiacenti(i);
		zTList<int> *l;
		for (l=Adia->First();!l->IsEnd();l=l->Next())
		{
			int *j=l->Read();
         if (color &&             // se abbiamo il vettore dei colori 
             color[*j]!=COL_BLUE) // possiamo utilizzare solo i nodi blu
             continue;
			if (D[i]+1 < D[*j])
			{
				T[*j]=i;
				D[*j]=D[i]+1;
				int *nj = new int;
				*nj=*j;
				if (!B[*nj])
				{
					Insieme->First()->Insert(nj);
					B[*nj]=true;
				}
			}
		}

	}
   zTList<int> *out=new zTList<int>(true);
   // Prepara la lista del cammino scorrendo il vettore dei padri
   zTList<int> *l;
   int *n;
   n=new int;
   *n=u;
   out->Insert(n);
   l=out->First();
   while(T[u] != r)
   {
      n=new int;
      *n=T[u];
      u=*n;
      l->Insert(n);
      l=l->Prev();
	}
   
	free(B);
	free(T);
	free(D);
	delete Insieme;
	return out;
}	
