/************************************************
**** GRAFO                                   ****
**** Implementazione di grafi non orientati. ****
****                                         ****
**** Alessandro Santuari   6 SI              ****
************************************************/

#include <stdio.h>
#include <string.h>
#include "ProgressBar.h"
#include "Robot.h"
#include "Grafo.h"


Grafo::Grafo()
{
	// inizializza a vuoto
	for (int i=0;i<MAX_VERTICES;i++)
		Vertici[i]=NULL;
	NumVertici=0;
	NumArchi=0;
}

Grafo::~Grafo()
{
	// svuota il grafo
	Reset();
}

int Grafo::Load(char *fn, bool show)
{
// carica il grafo da file
	
   char fname[1024];
   sprintf(fname,"%s%s",GRAPHS_DIR,fn);
   
	FILE *f;

	f=fopen(fname,"r");
	if (!f) return 1;

	int NV;
	fscanf(f,"%d\n",&NV);

   ProgressBar Avanzamento=ProgressBar("Sto caricando...",NV,20,30,'=');
	
	int i;
	for (i=1;i<=NV;i++)
		AddVertice();
	
	while (!feof(f))
	{
		int v1,v2;
		fscanf(f,"%d %d\n",&v1,&v2);
		AddArco(v1,v2);
      if (show)
         Avanzamento.Advance();
	}

	fclose(f);
	return 0;
}

int Grafo::Save(char *fn, bool show)
{
	// salva il grafo

   char fname[1024];
   sprintf(fname,"%s%s",GRAPHS_DIR,fn);

	FILE *f;
      
	f=fopen(fname,"w");
	if (!f) return 1;

   ProgressBar Avanzamento=ProgressBar("Sto salvando...",NumVertici,20,30,'=');

	fprintf(f,"%d\n",NumVertici);
	
	for (int i=1;i<=NumVertici;i++)
	{
		zTList<int> *Adiac,*l;
		Adiac=getAdiacenti(i);

		for (l=Adiac->First();!l->IsEnd();l=l->Next())
		{
			int *a=l->Read();
			if (*a>i)				// per evitare di scrivere un arco due volte
				fprintf(f,"%d %d\n",i,*a);
		}
      if (show) Avanzamento.Advance();
	}

	fclose(f);
	return 0;
}

int Grafo::SaveDOT(char *fname)
{
	// salva il grafo

	FILE *f;
      
	f=fopen(fname,"w");
	if (!f) return 1;

	fprintf(f,"Graph G {\n");
   //fprintf(f,"size=\"7.5,10\";ratio = compress;center=false;\n");
   fprintf(f,"size=\"7.5,10\";center=false;\n");
  	fprintf(f,"node [fontcolor=black,color=lightgray,shape=ellipse,style=filled,height=.1,width=.2,fontsize=10];\n");

	for (int i=1;i<=NumVertici;i++)
	{
		zTList<int> *Adiac,*l;
		Adiac=getAdiacenti(i);

		for (l=Adiac->First();!l->IsEnd();l=l->Next())
		{
			int *a=l->Read();
			if (*a>i)				// per evitare di scrivere un arco due volte
         {
            char Num1[32],Num2[32];
            sprintf(Num1,"%d",i);
            sprintf(Num2,"%d",*a);
				fprintf(f,"%s -- %s;\n",Num1,Num2);
         }
		}
	}
	fprintf(f,"}\n");
	fclose(f);
	return 0;
}

int Grafo::SaveDOTSpecial(char *fname,int RPos,bool *NodeKnown,bool **EdgeKnown,bool *NodeSature, int *Colors)
{
  	// salva il grafo

	FILE *f;
   int i;
   char Buf[1024];
      
	f=fopen(fname,"w");
	if (!f) return 1;

   // inizializzaioni del formato
	fprintf(f,"Graph G {\n");
	fprintf(f,"size=\"7.5,10\";ratio = compress;center=false;\n");
   fprintf(f,"node [shape=ellipse,height=.1,width=.2,fontsize=10];\n");
  	
   // formato di ogni vertice
  	for (i=1;i<=NumVertici;i++)
	{
      sprintf(Buf,"%d ",i);
      if (NodeKnown[i])                         // se e' un nodo esplorato...
      {
         if (NodeSature[i])                     // ed e' saturo
         {
            strcat(Buf,"[color=black");         // colore nero
            if (i==RPos)
               strcat(Buf,",fontcolor=white");
            else
               strcat(Buf,",fontcolor=black");
         }
         else
            strcat(Buf,"[color=gray90");        // se non saturo, colore grigio
      }
      else
         strcat(Buf,"[color=white,fontcolor=grey70"); // sconosciuto, grigio


      if (i==RPos)                     // se il robot e' presente
         strcat(Buf,",style=filled");  // casella riempita
      else                             // altrimenti
         strcat(Buf,",style=bold");    // solo cerchiato

      strcat(Buf,"];\n");
      fprintf(f,"%s",Buf);
   }
	
	for (i=1;i<=NumVertici;i++)
	{
		zTList<int> *Adiac,*l;
		Adiac=getAdiacenti(i);

		for (l=Adiac->First();!l->IsEnd();l=l->Next())
		{
			int *a=l->Read();
			if (*a>i)				// per evitare di scrivere un arco due volte
         {
            char Num1[32],Num2[32];
            sprintf(Num1,"%d",i);
            sprintf(Num2,"%d",*a);
				sprintf(Buf,"%s -- %s ",Num1,Num2);
            
            if (EdgeKnown[i][*a])               // se arco esplorato..
               strcat(Buf,"[color=black]");     // fallo nero
            else                                // altrimenti
               strcat(Buf,"[color=gray90]");    // fallo grigio

            strcat(Buf,";\n");
            fprintf(f,"%s",Buf);
         }
		}
	}
	fprintf(f,"}\n");
	fclose(f);
	return 0;

}

int Grafo::SaveIso(char *fn,int *Iso)
{
	// salva applicando la funzione isomorfismo

   char fname[1024];
   sprintf(fname,"%s%s",GRAPHS_DIR,fn);
	
   FILE *f;
	f=fopen(fname,"w");
	if (!f) return 1;

	fprintf(f,"%d\n",NumVertici);
	
	for (int i=1;i<=NumVertici;i++)
	{
		zTList<int> *Adiac,*l;
		Adiac=getAdiacenti(i);

		for (l=Adiac->First();!l->IsEnd();l=l->Next())
		{
			int *a=l->Read();
			if (*a>i)
				fprintf(f,"%d %d\n",Iso[i],Iso[*a]);
		}
	}
	fclose(f);

	// il file e' pero' disordinato... lo ricarico e salvo
	Grafo *G=new Grafo();
	G->Load(fname,false);
	G->Save(fname,true);
	delete G;

	return 0;
}


void Grafo::Reset()
{
	// svuota il grafo
	for (int i=0;i<=NumVertici;i++)
	{
		if(Vertici[i])
			delete Vertici[i];
	}
	NumVertici=0;
	NumArchi=0;
}

bool Grafo::VerticeOK(int id)
{
	// il vertice id e' corretto?
	if (id<1 || id >NumVertici
		|| Vertici[id]==NULL) return false;
	return true;
}

int Grafo::AddVertice()
{
	// crea nuovo vertice
	NumVertici++;
	Vertici[NumVertici]=new Vertice(NumVertici);
	return NumVertici;
}

bool Grafo::AddArco(int v1,int v2)
{
	if (!VerticeOK(v1) || !VerticeOK(v2)) 
		return false;		// errore nei vertici id!
	if (v1==v2)
		return false;		// coincidono!
		
	if (Vertici[v1]->IsAdiacente(v2) ||
		Vertici[v2]->IsAdiacente(v1))
		return false;		// sono gia presenti
		
	if (Vertici[v1]->AddAdiacente(v2) &&
		Vertici[v2]->AddAdiacente(v1))
	{
		// ok!
		NumArchi++;
		return true;
	}
	return false;
}

bool Grafo::EsisteArco(int v1,int v2)
{
	// verifica se esiste un arco
	if (!VerticeOK(v1) || !VerticeOK(v2)) 
		return false;
	bool v1_v2 = Vertici[v1]->IsAdiacente(v2);
	bool v2_v1 = Vertici[v2]->IsAdiacente(v1);

	if (v1_v2 && v2_v1)
		return true;		//esiste
	else if ((v1_v2 || v2_v1)==false)
		return false;
	else
	{
		printf("Errore! Esiste un arco orientato tra v(%d) e v(%d) !!\n",v1,v2);
		return false;
	}
}

zTList<int> *Grafo::getAdiacenti(int v)
{
	if (!VerticeOK(v)) 
		return NULL;
	return Vertici[v]->getAdiacenti();

}

bool Grafo::IsEqual(Grafo *G)
{/*
	Confronta il grafo corrente con quello passato in G, per vedere
	se sono isomorfi e con le stesse etichette per i vertici. 
*/
	if (NumVertici!=G->getNumVertici() ||	// se il n. vertici e' diverso
		NumArchi!=G->getNumArchi())			// o il n. di archi e' diverso
		return false;								// allora non e' uguale

	bool *MyV=(bool*)malloc(sizeof(bool)*(NumVertici+1));
	
	for (int i=1;i<=NumVertici;i++)			// per ogni vertice...
	{
		zTList<int> *MyAdia=getAdiacenti(i);   // prendi i suoi adicenti
		zTList<int> *GAdia=G->getAdiacenti(i); // e gli adiacenti di G
		zTList<int> *l;
		int QuantiMy=0;
		int QuantiG=0;

      // inizializza		
		for (int j=1;j<=NumVertici;j++)
			MyV[j]=false;

		for (l=MyAdia->First();!l->IsEnd();l=l->Next())
		{
			MyV[*(l->Read())]=true;
			QuantiMy++;
		}
		for (l=GAdia->First();!l->IsEnd();l=l->Next())
		{
			int *a=l->Read();
			if (!MyV[*a])	// controlla che entrambi abbiano
			{					// lo stesso insieme di adiacenza
				free (MyV);
				return false;
			}
			QuantiG++;
		}
		if (QuantiMy!=QuantiG) return false;
		QuantiMy=QuantiG=0;
	}

	free (MyV);
	return true;      // se siamo arrivati fino a qui allora erano uguali!
}

void Grafo::Randomize(int Nodi,int Archi)
{/*
   Questa funzione crea un grafo casuale connesso utilizzando in numero
   di nodi ed il numero di archi specificato.
*/
	int i;
	int NumConnessi;
	long ArchiInseriti=0;

   ProgressBar Avanzamento=ProgressBar("Genero il grafo...",Archi,30,30,'=');

	Reset(); // resetta il grafo
  	
	int *Connessi=(int*)malloc(sizeof(int)*(Nodi+1));

   // crea tutti i vertici	
	for (i=1;i<=Nodi;i++)
	{
		AddVertice();
		Connessi[i]=0;
	}
	
	Connessi[1]=1;
	NumConnessi=1;

	do
	{
		int u,v;
		do
		{
         // scegli un nodo appartenente al gruppo dei connessi
         u=Connessi[(rand()%NumConnessi)+1];
         
         // scegli un nodo a caso
  			v=(rand()%Nodi)+1;         
			if (NumConnessi<Nodi)   // finche' ci sono nodi sconnessi
			{
            // v deve essere un nodo sconnesso!
				for(int j=1;j<=NumConnessi;j++)
				{
					if (Connessi[j]==v)
					{
						v=(rand()%Nodi)+1;
						j=0;
					}
				}
			}
				
		} while (u==v || !AddArco(u,v)); // finche' u e' diverso da v e posso inserire l'arco

      // ok, inserire l'arco tra u e v
      
		ArchiInseriti++;
		bool GiaInserito=false;
      // controllo se devo inserire v nell'insieme dei nodi connessi
		if (NumConnessi<Nodi)   // devo controllare...
		{
			for(int j=1;j<=NumConnessi;j++)
			{
				if (Connessi[j]==v)
				{
					GiaInserito=true;
					break;
				}
			}
		}
		else  // e' sicuramente gia inserito
			GiaInserito=true;
         
		if (!GiaInserito) // se non e'gia' inserito
		{
         // inserisco v nell' insieme connessi
			NumConnessi++;
			Connessi[NumConnessi]=v;
		}
      
      // visualizza la barra d'avanzamento ogni 30 archi inseriti
      
       Avanzamento.Advance();
      
	} while (ArchiInseriti<Archi);   // finche' non ho inserito tutti gli archi


	printf("\n");
			
	free (Connessi);
       
}



