/************************************************
**** GRAFO                                   ****
**** Implementazione di grafi non orientati. ****
****                                         ****
**** Alessandro Santuari   6 SI              ****
************************************************/

#if !defined(GRAFO_H_INCLUDED)
#define GRAFO_H_INCLUDED

#include "zTList.h"
#include "Vertice.h"

#define		MAX_VERTICES	10000

class Grafo  
{
public:
	Grafo();
	virtual ~Grafo();

	// 'svuota'	tutto il grafo
	void Reset();
	
	// carica un grafo da file
	int  Load(char *fname, bool show);
	
	// salva il grafo su un file
	int  Save(char *fname, bool show);
	
	// salva il grafo su un file, applicando una funzione di isomorfismo
	int SaveIso(char *fname, int *iso);

	// salva il grafo nel formato dei programmi DOT e NEATO
	int SaveDOT(char *fname);

   // salva il grafo nel formato dei programmi DOT e NEATO, utilizzando altre
   // informazioni (utile per analizzare il comportamento dell'algoritmo)
	int SaveDOTSpecial(char *fname,int RPos,bool *NodeKnown,bool **EdgeKnown, bool *NodeSature, int *Colors);

	// crea un grafo connesso randomizzato con il numero
	// di Nodi e Archi specificato
	void Randomize(int Nodi,int Archi);

	int getNumVertici() {return NumVertici;}
	int getNumArchi  () {return NumArchi  ;}

	// l'id del vertice e' valido?
	bool VerticeOK(int id);

	// crea un nuovo nodo
	int AddVertice();
	
	// aggiunge un arco tra il nodo v1 e il nodo v2
	bool AddArco(int v1,int v2);

	// esiste un arco tra v1 e v2?
	bool EsisteArco(int v1,int v2);
	
	// restituisci l'insieme d'adiacenza del vertice v
	zTList<int> *getAdiacenti(int v);

	// i due grafi corrispondono?
	// non viene verificato un eventuale isomorfismo, i vertici
	// dei grafi devono avere le stesse etichette
	bool IsEqual(Grafo *G);
	
private:
	Vertice *Vertici[MAX_VERTICES];
	int NumVertici;
	int NumArchi;
};

#endif // !defined(GRAFO_H_INCLUDED)
