/************************************************
**** PANAITEBOT                              ****
**** Robot che implementa l'algoritmo di     ****
**** esplorazione di Panaite e Pelc e al sua ****
**** versione migliorata                     ****
****                                         ****
**** Alessandro Santuari   6 SI              ****
************************************************/

#include <stdio.h>
#include "PanaiteBOT.h"

PanaiteBOT::PanaiteBOT(RInterface *ri, bool avanzato):BOT(ri)	// COSTRUTTORE
{
	// eredito dalla classe BOT
	// inizializzazioni
	for (int i=0;i<MAX_VERTICES;i++)
	{
		Parent[i]=PARENT_UNASSIGNED;
		Color [i]=COL_WHITE;
	}
	InternalPenalty=0;
	ExternalPenalty=0;
   Avanzato=avanzato;
}

PanaiteBOT::~PanaiteBOT()
{
}

bool PanaiteBOT::IsSaturated()
{
	// Un vertice e' saturo se non ci sono piu' archi inesplorati adiacenti
	return !RI->ExistUnexploredEdge();
}

bool PanaiteBOT::IsAlmostSaturated()
{
	// Un vertice e' quasi saturo se esiste al massimo un arco inesplorato
	return (RI->HowManyUnexploredEdge()<=1);
}


void PanaiteBOT::Saturate(int v, bool Almost)
{/*
	Questa funzione guida il robot attraverso il grafo partendo ed infine tornando
	al nodo v. Quando la funzione termina, il nodo v e' saturo, cioe' non ci sono piu'
	archi inesplorati che partono da v.
*/	
	int u;

   zTList<int> *path=NULL;			
   
	Color[v]=COL_BLUE;
	u=v;
  
  	while (1)
	{
      if (u==v)
      {
         // verifico la condizione di terminazione d'arresto
         if (Almost && IsAlmostSaturated()) break;
         if (!Almost && IsSaturated())      break;
      }
      
		if (IsSaturated()==false)	// se non e' saturo
		{
			// muovi il robot lungo una via inesplorata!
			int OldPos=getMyPos();
			RI->MoveToUnexplored();
			// notifica lo spostamento
			JustMoved(OldPos);
			u=getMyPos();
			Color[u]=COL_BLUE;
         
         if (path)   // se avevamo memorizzato un percorso di ritorno...
         {           // adesso non serve piu'
            delete path;
            path=NULL;
         }
		}
		else
		{
			// siamo bloccati!! Dobbiamo tornare indietro lungo il minimo cammino che ci porta a
			// v, passando solo sui nodi colorati di blu.
         int s;

         if (!path)              // se non abbiamo gia' memorizzato il percorso,
            path=FindPathTo(v, Color);  // cercalo!
   
         // estrai il prossimo nodo dal cammino
         s=*(path->First()->Read());
         path->First()->Remove();
         
         if (path->IsEmpty()) // se e' vuoto non serve piu'
         {
            delete path;path=NULL;
         }

			// abbiamo fatto una mossa penalita'
			Color[u]=COL_RED;
            InternalPenalty++;
			u=s;
			int OldPos=getMyPos();
			// torna indietro
			RI->MoveToExplored(MyMap_to_RI(u));
			// notifica lo spostamento
			JustMoved(OldPos);
		}
		printf ("%d ",MyMap_to_RI(getMyPos()));
		fflush(stdout);
  
	}

	// ok, siamo tornati a v che adesso e' saturo. Tutti i nodi colorati di blu
	// tornano bianchi
	for (int i=1;i<=MyMap->getNumVertici();i++)
		if (Color[i]==COL_BLUE)
			Color[i]=COL_WHITE;

   if (path) // libera la memoria
   {
      delete path;
      path=NULL;
   }

         
   printf("\n");
}

void PanaiteBOT::_explore()
{/*
	Al termine di questa funzione il robot avra' esplorato ogni vertice ed ogni arco
	del grafo ambiente. Inoltre ne avra' costruito la mappa sotto forma di grafo isomorfo.
*/
	int v,r;
	r=getMyPos();
	Parent[r]=PARENT_NULL;
	v=r;

	printf ("%d\n",MyMap_to_RI(getMyPos()));

	while (1)
	{
		if (IsSaturated()==false)	// se il nodo non e' saturo...
      {
     		printf (" SATURO %d:\n",MyMap_to_RI(v));
 			Saturate(v,Avanzato);				// allora satura!
      }
      
      if (Avanzato)
      {
         if (IsSaturated()==false)
         {
            // abbiamo un solo arco inesplorato
        	  int OldPos=getMyPos();
      	  RI->MoveToUnexplored();
      	  JustMoved(OldPos);
           printf (" %d: ",MyMap_to_RI(getMyPos()));fflush(stdout);
           Parent[getMyPos()]=OldPos;
           v=getMyPos();
           continue;
         }
      }
           	
      // per definizione della 'Saturate', il robot ora si trova ancora
      // al vertice v e ha esplorato tutti gli archi che partono da v

      // adesso dobbiamo muovere il robot cercando nodi non saturi

      int a;
      a=FindUnassigned(v);
      if (a>0)
      {
			Parent[a]=v;
			v=a;
      }
      else
      {
         // sono bloccato... non rimane che tornare indietro
         // uso lo spanning tree per tornare indietro
         if (!Avanzato)
         {
   			v=Parent[v];
            if (v==PARENT_NULL) // finito l'esplorazione???
               break;           // esci dal while
         }
         else
         {
            // miglioramento
            
            int n=v;
            
            while(Parent[n]!=PARENT_NULL && FindUnassigned(n)<0)
               n=Parent[n];
            
            if (n==v)
               break;
//            printf("Meglio %d\n",MyMap_to_RI(n));
            // riloca su n con cammino minimo
            
            zTList<int> *Path= FindPathTo(n,NULL);
      		zTList<int> *l;
            // spostare il robot lungo il cammino
      		for (l=Path->First();!l->IsEnd();l=l->Next())
      		{
		      	int *a=l->Read();
      			RI->MoveToExplored(MyMap_to_RI(*a));
               ExternalPenalty++;
           	  JustMoved(*a);
               v=*a;
		      	printf("*%d ",RI->getRobotPos());
      			fflush(stdout);
      		}
		      delete Path;

            if (v==PARENT_NULL) // finito l'esplorazione???
               break;           // esci dal while
            else
               continue;
               
            
            
         }
         
      }
         
      // effetua lo spostamento
      
      printf (" %d: ",MyMap_to_RI(v));fflush(stdout);
	  int OldPos=getMyPos();
	  RI->MoveToExplored(MyMap_to_RI(v));
	  JustMoved(OldPos);
	  // penalita' esterna!
	  ExternalPenalty++;
	}
	printf("\nPenalita':\n");
	printf("   EXT = %d , MAX = %d\n",ExternalPenalty,-2+MyMap->getNumVertici()*2);
	printf("   INT = %d , MAX = %d\n",InternalPenalty,MyMap->getNumVertici());
	printf("   TOT = %d , MAX = %d\n",
		ExternalPenalty+InternalPenalty,-2+MyMap->getNumVertici()*3);

}
          

int PanaiteBOT::FindUnassigned(int v)
{
   zTList<int> *Adia=MyMap->getAdiacenti(v);
	zTList<int> *l;

	for (l=Adia->First();!l->IsEnd();l=l->Next())
	{
		int *a=l->Read();
      // cerca un nodo non presente nello spanning tree
		if (Parent[*a]==PARENT_UNASSIGNED)  
		{
         // trovato!
         return *a;
		}
	}
   return -1;    
}