/************************************************
**** RINTERFACE                              ****
**** L'interfaccia che deve usare il robot   ****
**** per interagire con l'ambiente.          ****
****                                         ****
**** Alessandro Santuari   6 SI              ****
************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>

#include "RInterface.h"
#include "ProgressBar.h"

RInterface::RInterface(Grafo *env, int startPos, bool avversario, int sc, char *dir)
{/*
	Crea l'interfaccia che sara' utilizzata dal robot per raccogliere
	informazioni sull'ambiente e per spostarsi
	
	env        --> il grafo ambiente
	startPos   --> la posizione inizale del robot
	avversario --> avversario manuale o automatico??
   sc         --> ogni quante mosse scattare la foto
   dir        --> directory dove salvare le foto
*/
	Environ=env;
	Avversario=avversario;
   ScattaFoto=sc;
   if (startPos<=0 || startPos>env->getNumVertici())
   {
      printf("Vertice di partenza non valido!\n");
      startPos=1;
   }
	RPos=startPos;
	Moves=0;
   NumSnapshot=0;
	int dim=Environ->getNumVertici()+1;
	
	// alloca la matrice degli archi conosciuti dal robot
	EdgeKnown = (bool**)malloc( sizeof(bool*)*dim);
	for (int i=1;i<dim;i++)
		EdgeKnown[i] = (bool*)malloc( sizeof(bool)*dim);

	// alloca il vettore dei nodi conosciuti
	NodeKnown=(bool*)malloc( sizeof(bool)*dim);

	// inizializza tutto a false
	for (int y=1;y<dim;y++)
	{
		NodeKnown[y]=false;
		for (int x=1;x<dim;x++)
			EdgeKnown[y][x]=false;
	}

	NodeKnown[RPos]=true;	// conosco solo il vertice di partenza

   strcpy(SnapDir,dir);
   if (ScattaFoto) SaveSnapshot();
}

RInterface::~RInterface()
{
	// libera la memoria
	int dim=Environ->getNumVertici()+1;

	for (int i=1;i<dim;i++)
		free(EdgeKnown[i]);
	free(EdgeKnown);
	free(NodeKnown);
}

bool RInterface::ExistUnexploredEdge()
{
	// il robot ci chiede se nella sua posizione esiste un arco
	// inesplorato. 

	zTList<int> *Unexplored=MakeUnexploredList();

	bool ris=true;
	if (Unexplored->IsEmpty())
		ris=false;

	delete Unexplored;
	
	return ris;
}

bool RInterface::ExistUnexploredEdgeHere(int v)
{
	// il robot ci chiede se al nodo v esiste un arco
	// inesplorato. Il robot deve aver gia' esplorato v!!
   
   if (NodeKnown[v]==false)
   {
      printf("\nATTENZIONE: Il robot tenta di acquisire informazioni\n");
      printf("            su nodi sconosciuti!\n");
      return false;
   }

	zTList<int> *Unexplored=MakeNodeList(v,false);

	bool ris=true;
	if (Unexplored->IsEmpty())
		ris=false;

	delete Unexplored;
	
	return ris;
}


int RInterface::HowManyUnexploredEdge()
{
	// il robot ci chiede quanti archi inesplorati
	// partono dalla sua posizione

	int count=0;
	zTList<int> *Unexplored=MakeUnexploredList();
	zTList<int> *l;
	for (l=Unexplored->First();!l->IsEnd();l=l->Next())
		count++;

	delete Unexplored;
	return count;
}

zTList<int> *RInterface::MakeUnexploredList()
{/*
	Funzione PRIVATA!! Non puo' essere invocata dal robot, perche' esso NON
	deve essere a conoscenza dei vertici inesplorati!

	Prepara una lista dei vertici inesplorati adiacenti alla posizione del robot.
*/
	return MakeNodeList(RPos,false);
}

zTList<int> *RInterface::MakeExploredList()
{
	// Prepara una lista dei vertici esplorati adiacenti alla posizione del robot.
	return MakeNodeList(RPos,true);
}

zTList<int> *RInterface::MakeNodeList(int v,bool explored)
{/*
	Prepara una lista dei vertici inesplorati o esplorati
	adiacenti al vertice v
*/
	zTList<int> *out=new zTList<int>(true);

	// Intanto vediamo i vertici adiacenti alla sua posizione
	zTList<int> *Adia=Environ->getAdiacenti(v);
	zTList<int> *l;
	for (l=Adia->First();!l->IsEnd();l=l->Next())
	{
		int *a=l->Read();
		if (EdgeKnown[v][*a]==explored)	// arco corrispondente al
		{												// criterio scelto!
			int *newa = new int;
			*newa=*a;
			out->Insert(newa);
		}
	}
	return out;
}

bool RInterface::MoveToUnexplored()
{/*
	Ha il compito di muovere il robot lungo un arco inesplorato.
	L'arco e' scelto arbitrariamente.
	Se l'avversario e' automatico, viene scelto un arco casualmente...
	Altrimenti viene chiesto all' utente di agire come l'avversario
*/
	if (ExistUnexploredEdge()==false)
		return false;	// errore! sono tutti esplorati!

	zTList<int> *Unexplored,*l;
	int NumArchi=0;

	Unexplored=MakeUnexploredList();
	
	// conto gli archi
	for (l=Unexplored->First();!l->IsEnd();l=l->Next())
		NumArchi++;
		
	int *Possibilita=(int*)malloc(sizeof(int)*NumArchi);
	
	l=Unexplored->First();
	int i;
	for (i=0;i<NumArchi;i++)
	{
		int *a=l->Read();
		Possibilita[i]=*a;
		l=l->Next();
	}
	
	int To;
	if (Avversario)
	{
		// Avversario manuale
		
		if (NumArchi>1)	// esiste piu' di una scelta?
		{
			bool ok=false;
			// stampa le scelte possibili e aspetta l'input dall'utente
			do
			{
				printf("Muovi: ");
				for (i=0;i<NumArchi;i++)
				{
					int n=Possibilita[i];
					printf("%d ",n);
				}
				printf("\n");
				scanf("%d",&To);
				// verico che sia corretto
				for (i=0;i<NumArchi;i++)
				{
					if (To==Possibilita[i])
					{
						ok=true;
						break;
					}
				}
				if (!ok)
					printf("Errore!\n");
			} while (!ok);
		}
		else 							// se esiste una sola possibilita'
			To=Possibilita[0];	// scegli quella
	}
	else								// se l'avversario e' automatico...
		To=Possibilita[rand()%NumArchi];	// scegli una via casuale
	
	// Muovi il robot!
	_move(To);	

	delete Unexplored;

	return true;
}

bool RInterface::MoveToExplored(int v)
{/*
	Ha il compito di muovere il robot lungo un arco esplorato.
	Verifichiamo che veramente il robot conosca la destinazione e che 'v'
	sia adiacente al robot (cioe' il robot non deve barare)
*/
	zTList<int> *Explored,*l;
	
	if (v==RPos) return true;	//rimane fermo

	Explored=MakeExploredList();
	if (Explored->IsEmpty())
		return false;	// non ha esplorato niente!

	if (Environ->VerticeOK(v)==false)
		return false;	// vertice non valido!

	for (l=Explored->First();!l->IsEnd();l=l->Next())
	{
		// verifico che v sia presente
		int *a=l->Read();
		if (*a==v)
		{
			// trovato! muovi il robot
			_move(v);
			delete Explored;
			return true;
		}
	}

	delete Explored;

	printf("Il robot ha tentato di muoversi al vertice %d che non e' exp o non adiacente\n",v);

	return false;
}

void RInterface::_move(int to)
{/*
	Funzione privata di basso livello che muove il robot nel vertice 'to'.
	Si presume che vertice specificato sia corretto
*/
	EdgeKnown[RPos][to]=true;
	EdgeKnown[to][RPos]=true;
	NodeKnown[to]=true;
	RPos=to;
	// aumenta il contatore delle mosse
	Moves++;
   if (ScattaFoto&&
      !(Moves%ScattaFoto)) SaveSnapshot();
}

void RInterface::Finish()
{
   char Buf[512];
   // Il robot ha finito l'esplorazione, faccio l'ultima fotografia
   if (ScattaFoto==0) return;
   if (Moves%ScattaFoto) 
      SaveSnapshot();
}

void RInterface::SaveSnapshot()
{/*
   Salva una fotografia della situazione corrente
*/
   char FileDOT[1024];
   char Cmd[1024];

   if (NumSnapshot==0)                       // se e' il primo
   {
      sprintf(Cmd,"rm %s*.* >& /dev/null",SnapDir);
      system(Cmd);                           // fare pulizia
   }

   NumSnapshot++; // nuovo snapshot

   sprintf(FileDOT,"%ssnap%0.4d.dot",SnapDir,NumSnapshot); // nome del file DOT/NEATO

   // Prepara un vettore booleano per sapere se un nodo e' saturo
   bool *NodeSature=(bool*)malloc(sizeof(bool)*(Environ->getNumVertici()+1));
   for (int i=1;i<=Environ->getNumVertici();i++)
   {
      zTList<int> *l=MakeNodeList(i,false);
      NodeSature[i]=l->IsEmpty();
   }
   // Salva il file in formato .dot, contenente anche il layout
   Environ->SaveDOTSpecial(FileDOT,RPos,NodeKnown,EdgeKnown,NodeSature,NULL);
   free(NodeSature);
}

void RInterface::SviluppaFoto()
{
   char Cmd[1024];
   char FileDOT[1024];
   char FilePS[1024];
   char FileGIF[1024];
   char FilePPM[1024];
   
   if (ScattaFoto==0) return;

   DIR *FotoDir=opendir(SnapDir);
   dirent *strdir;
   
   if (!FotoDir)
   {
      printf("ERRORE: Non posso creare le immagini gif!\n");
      fflush(stdout);
      return;
   }

   int NSnaps=0;
   ProgressBar Avanzamento=ProgressBar("Sto preparando tutte le foto...",NumSnapshot*2,1,30,'=');
   
   while((strdir=readdir(FotoDir))!=NULL) // per ogni file della directory...
   {
      // controllo se e' un .dot
      if (strstr(strdir->d_name,".dot")==NULL || 
          strstr(strdir->d_name,"snap")==NULL) continue;
          
      NSnaps++;
      
      sprintf(FileDOT,"%ssnap%0.4d.dot",SnapDir,NSnaps); // nome del file DOT/NEATO
      sprintf(FilePS ,"%ssnap%0.4d.ps" ,SnapDir,NSnaps); // nome del file PS
      sprintf(FileGIF,"%ssnap%0.4d.gif",SnapDir,NSnaps); // nome del file GIF
      sprintf(FilePPM,"%ssnap%0.4d.ppm",SnapDir,NSnaps); // nome del file PPM

      // prepara la fotografia con NEATO
      sprintf(Cmd,DRAWGRAPH_PRG,"neato",FileDOT,FilePS);
//      sprintf(Cmd,DRAWGRAPH_PRG,"dot",FileDOT,FilePS);
      system(Cmd);
      
      Avanzamento.Advance();

      // elimina il file DOT
      unlink(FileDOT);

      // trasforma il ps in gif
      sprintf(Cmd,CONVERT_PRG,FilePS,FileGIF);
      system(Cmd);
   
      Avanzamento.Advance();
      
      // elimina il file PS e PPM (temporaneo)
//      unlink(FilePS);
      unlink(FilePPM);

   }
   closedir(FotoDir);
 
   // visualizzare tutti i fotogrammi!!
   printf("-> Visualizzo i fotogrammi...\n");fflush(stdout);
   sprintf(Cmd,SHOWGIF_PRG,SnapDir);
   system(Cmd);

}
