/************************************************
**** RINTERFACE                              ****
**** L'interfaccia che deve usare il robot   ****
**** per interagire con l'ambiente.          ****
****                                         ****
**** Alessandro Santuari   6 SI              ****
************************************************/

#if !defined(RINTERFACE_H_INCLUDED)
#define RINTERFACE_H_INCLUDED

#include "Robot.h"
#include "Grafo.h"

class RInterface  
{		// l'interfaccia utilizzata dal robot per comunicare con l'ambiente
public:
	RInterface(Grafo *Environ, int startPos, bool avversario, int scattafoto, char *dir);
	virtual ~RInterface();

	int getRobotPos() {return RPos;}	   // posizione attuale del robot
	int getMoves() {return Moves;}	   // numero di mosse effettuate
	bool ExistUnexploredEdge();		   // esiste almeno un arco inesplorato?
  	bool ExistUnexploredEdgeHere(int v);// esiste almeno un arco inesplorato in v?
	int  HowManyUnexploredEdge();		   // quanti archi inesplorati?
	bool MoveToUnexplored();			   // muovi il robot su un arco inesplorato
	bool MoveToExplored(int v);		   // muovi verso uno specifico vertice
	      									// utilizzando un arco esplorato

   void Finish();                   // il robot ha finito l'esplorazione
   void SviluppaFoto();             // crea un immagine gif di tutte le foto
   
	// prepara una lista di vertici adiacenti (esplorati)
	zTList<int> *MakeExploredList(); 
   
private:

	// il grafo ambiente, non accessibile direttamente dal robot
	Grafo *Environ;
	
	// posizione (vertice) attuale del robot
	int RPos;
	
	// true --> avversario manuale,  false--> avversario automatico
	bool Avversario;

   // ogni quante mosse deve scattare una foto (0=mai)
	int ScattaFoto;

	// matrice degli archi conosciuti dal robot
	bool **EdgeKnown;
	
	// matrice dei nodi conosciuti dal robot
	bool *NodeKnown;
		
	//numero di mosse effettuato dal robot
	int Moves;

   // numero di foto fatte
   int NumSnapshot;
   
   // directory dove salvare le foto
   char SnapDir[128];
   
	// fai una lista di vertici esplorati (o inesplorati)
	zTList<int> *MakeNodeList(int v,bool explored);
	
	// fai una lista di vertici inesplorati
	zTList<int> *MakeUnexploredList();

	// muovi il robot!
	void _move(int to);

   // scatta una foto della mossa
   void SaveSnapshot();


};

#endif // !defined(RINTERFACE_H_INCLUDED)
