/************************************************
**** ROBOT                                   ****
**** Modulo principale.                      ****
**** Stampa menu e esegue le richieste       ****
**** dell'utente.                            ****
****                                         ****
**** Alessandro Santuari   6 SI              ****
************************************************/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>

#include "Robot.h"
#include "DfsBOT.h"
#include "GreedyBOT.h"
#include "PanaiteBOT.h"
#include "RInterface.h"
#include "Grafo.h"
#include "Menu.h"

#define		ALG_NONE		   0
#define		ALG_DFS			1		// algoritmo selezionato: DFS
#define		ALG_GREEDY		2		// algoritmo selezionato: GREEDY
#define		ALG_PANAITE		3		// algoritmo selezionato: PANAITE&PELC
#define		ALG_PANAITE_2  4     // algoritmo selezionato: PANAITE&PELC almost satura

#define		AVV_NONE		   0
#define		AVV_AUTO		   1     // avversario: AUTOMATICO
#define		AVV_MANUAL		2     // avversario: MANUALE

static int  QualeAlgoritmo;		// l'algoritmo attuale selezionato
static int  QualeAvversario;		// la modalita' avversario utilizzata
static int  NodoPartenza;			// il nodo di partenza per l'esplorazione
static int  ScattaFoto;          // ogni quante mosse devo scattare una foto??

static Menu MainMenu;				// menu principale
static Menu AlgMenu;					// menu scelta algoritmo
static Menu AvvMenu;					// menu scelta avversario
static Menu GrafoMenu;				// menu cambia grafo
static Menu FotoMenu;				// menu scatta foto

static char GrafoLoaded[512];		// informazioni sull'ambiente attuale
static Grafo *G=NULL;				// il grafo ambiente

void Init()
{
	// Inizializzazioni varie...
	
	// inizializza il generatore di numeri pseudocasuali
	srand(time(NULL));
	
	// prepara il menu principale
	MainMenu.AddOpt("Cambia grafo");
	MainMenu.AddOpt("Cambia algortimo");
	MainMenu.AddOpt("Cambia modalita' avversario");
   MainMenu.AddOpt("Cambia modalita' foto");
 	MainMenu.AddOpt("Visualizza grafo (DOT&NEATO)");
	MainMenu.AddOpt("Esegui!");
	MainMenu.AddOpt("*Uscita*");
	
	// prepara il menu scelta algoritmo
	AlgMenu.AddOpt("Depth First Search (DFS)");
	AlgMenu.AddOpt("Greedy");
	AlgMenu.AddOpt("Panaite & Pelc");
  	AlgMenu.AddOpt("Panaite & Pelc (Avanzato)");
	AlgMenu.AddOpt("*Lascia invariato*");
	
	// prepara il menu scelta avversario
	AvvMenu.AddOpt("Automatico (scelta casuali)");
	AvvMenu.AddOpt("Manuale    (sarai l'avversario!)");
	AvvMenu.AddOpt("*Lascia invariato*");
	
	// prepara il menu cambia grafo
	GrafoMenu.AddOpt("Carica un grafo da file");
	GrafoMenu.AddOpt("Genera grafo casualmente");
	GrafoMenu.AddOpt("*Annulla*");

   // prepara il menu scatta foto
	FotoMenu.AddOpt("Non scattare foto");
	FotoMenu.AddOpt("Scatta una foto ogni 200 mosse");
   FotoMenu.AddOpt("Scatta una foto ogni 100 mosse");
   FotoMenu.AddOpt("Scatta una foto ogni  50 mosse");
   FotoMenu.AddOpt("Scatta una foto ogni  10 mosse");
   FotoMenu.AddOpt("Scatta una foto ogni   5 mosse");
   FotoMenu.AddOpt("Scatta una foto ogni   2 mosse");
   FotoMenu.AddOpt("Scatta una foto ogni mossa!");
	FotoMenu.AddOpt("*Annulla*");

	strcpy(GrafoLoaded,"NESSUNO");
	QualeAlgoritmo=ALG_PANAITE;
	QualeAvversario=AVV_AUTO;
	NodoPartenza=0;
   ScattaFoto=false;
}

int FotoMenu_Mosse(int nmenu)
{
   switch (nmenu)
   {
   case 2: return 200;break;
   case 3: return 100;break;
   case 4: return  50;break;
   case 5: return  10;break;
   case 6: return   5;break;
   case 7: return   2;break;
   case 8: return   1;break;
   default: return 0;
   };

}

void PrintStatus()
{/*
	Stampa le impostazioni correnti
*/
	char Buf[512];
	printf("\n");
	
	switch(QualeAlgoritmo)
	{
		case ALG_DFS:
			strcpy(Buf,"DFS");break;
		case ALG_GREEDY:
			strcpy(Buf,"GREEDY");break;
		case ALG_PANAITE:
			strcpy(Buf,"PANAITE");break;
      case ALG_PANAITE_2:
			strcpy(Buf,"PANAITE 2");break;
	}
	printf("Grafo attuale : %s\n",GrafoLoaded);
	printf("Algoritmo     : %s\n",Buf);

	switch(QualeAvversario)
	{
		case AVV_AUTO:
			strcpy(Buf,"Automatico");break;
		case AVV_MANUAL:
			strcpy(Buf,"Manuale");break;
	}
	printf("Avversario    : %s\n",Buf);
   if (ScattaFoto!=0)
   {
      if (ScattaFoto==1)
         printf("Foto ogni     : 1 mossa\n");
      else
         printf("Foto ogni     : %d mosse\n",ScattaFoto);
   }
   else
      printf("Foto          : NESSUNA\n");
   
	printf("\n");
}

bool CaricaGrafoFile()
{/*
	Tenta di caricare un grafo da file.
	Restituisce true in caso di successo, false altrimenti
*/
	char Buf[1024];
		
	if (G)					// se era gia' presente un grafo...
		delete G;			// bye bye
	G=new Grafo();			// nuovo grafo (vuoto)
	
	sprintf(Buf,"ls %s",GRAPHS_DIR);
	system(Buf);
	
	printf("\nInserisci il nome del file: ");
	fgets(Buf,1024,stdin);
   if (strlen(Buf)<=1) return false;

	// togli il \n
	Buf[strlen(Buf)-1]='\0';
	printf("Tento di caricare il grafo...\n");
	
	if (G->Load(Buf,true))		// carica il grafo da file
	{
		// errore
		printf("ERRORE! Non posso caricare il grafo '%s'!\n",Buf);
		// distruggi il grafo
		delete G;
		G=NULL;
		return false;
	}
	// caricato correttamente!
	sprintf(GrafoLoaded,"FILE->'%s' |V|=%d |E|=%d",
		Buf,G->getNumVertici(),G->getNumArchi());
	return true;
}

bool GeneraGrafo()
{/*
	Genera un grafo connesso casuale. L'utente specifica il numero di nodi
	e il numero di archi.
*/
	char Buf[1024];
	if (G)				// se era gia' caricato un grafo
		delete G;		// bye bye
	G=new Grafo();		// nuovo grafo
	
	int Nodi=0;
	int Archi=0;
	do
	{
		printf("Inserire il numero di nodi: ");
		fgets(Buf,1024,stdin);
		Nodi=atoi(Buf);
	} while (Nodi<=1 || Nodi>MAX_VERTICES);

	int min,max;
	do
	{
		min=Nodi-1;					// minimo numero di archi perche sia connesso
		max=(Nodi*(Nodi-1))/2;	// massino num di archi (grafo completo)
		printf("Inserire il numero di archi (min %d, max %d): ",min,max);
		fgets(Buf,1024,stdin);
		Archi=atoi(Buf);
	} while (Archi<min || Archi>max);
	printf("Genero il grafo, attendere...\n");
	
	G->Randomize(Nodi,Archi);	// crea il grafo
	
	sprintf(GrafoLoaded,"(RANDOM) |V|=%d |E|=%d",
			G->getNumVertici(),G->getNumArchi());
   fflush(stdout);
	
	return true;		
}

void VediGrafo()
{
   char Buf[1024];
   //sprintf(Buf,"~/gv1.7c/bin/neato -Tps -Gepsilon=.0001 temp.nto -o temp.ps&");
   //system("ghostview temp.ps &");
 
   // Lancia il programma 'dot' per creare il disegno del grafo a partire dal file "temp.dot"
   printf("-> Sto disegnando il grafo con il programma DOT...\n");fflush(stdout);
   sprintf(Buf,DRAWGRAPH_PRG,"dot","temp/temp.dot","temp/dot.ps");
   system(Buf);

   // Lancia il programma 'neato' per creare il disegno del grafo a partire dal file "temp.dot"
   printf("-> Sto disegnando il grafo con il programma NEATO...\n\n");fflush(stdout);
   sprintf(Buf,DRAWGRAPH_PRG,"neato","temp/temp.dot","temp/neato.ps");
   system(Buf);
   
   // Lancia il programma per visualizzare il ps "dot.ps"
   printf("-> Visualizzo il grafo prodotto con DOT...\n");fflush(stdout);
   sprintf(Buf,SHOWPS_PRG,"temp/dot.ps");
   system(Buf);

   // Lancia il programma per visualizzare il ps "neato.ps"
   printf("-> Visualizzo il grafo prodotto con NEATO...\n");fflush(stdout);
   sprintf(Buf,SHOWPS_PRG,"temp/neato.ps");
   system(Buf);

}


int main (int argc, char *argv[])
{
   // Inizializzazioni 
	Init();

	int scelta;             // scelta dei menu
	bool exit=false;        // condizione d'uscita?
	BOT *TheBot;            // Il robot esploratore
	RInterface *TheRI;      // l'interfaccia
	bool avvmode;           // avversario manuale?
   char Buf[256];
	bool ok; 

	Grafo *RobotMap=NULL;   // Mappa del robot

	while(!exit)
	{
	
		PrintStatus();             // stampa le impostazioni correnti
		scelta=MainMenu.Select();  // chiede una scelta nel menu principale

		switch(scelta)
		{
			case 1:	//---------------- CAMBIA GRAFO -----------------
				printf("--- Cambia il grafo ambiente ---\n");
				scelta=GrafoMenu.Select();			
				ok=false;
				if (scelta==3) break;   // lascia invariato
				if (scelta==1)          // carica da file
					ok=CaricaGrafoFile();
				if (scelta==2)          // genera grafo casuale
					ok=GeneraGrafo();
				if (ok)                 // se il grafo e' stato creato
				{
					// salvo il nuovo grafo in temp.txt (formato del mio programma)
					G->Save("temp.txt",false);
               // salvo il nuovo grafo in temp.dot (formato del programma DOT)
               G->SaveDOT("temp/temp.dot");
               // stampo info
              	printf("OK!  |V|=%d |E|=%d \n",G->getNumVertici(),G->getNumArchi());
          
					do // scelta del punto di partenza del robot
					{
                  char Buf[32];
						printf("Scegli il nodo di partenza: ");fflush(stdout);
                  fgets(Buf,32,stdin);
                  NodoPartenza=atoi(Buf);
					} while (NodoPartenza<1 || NodoPartenza>G->getNumVertici());
               // aggiungi alle info il nodo di partenza
					sprintf(GrafoLoaded,"%s r=%d",GrafoLoaded,NodoPartenza);
				}
				break;
			case 2:	//-------------- CAMBIA ALGORITMO ---------------
				printf("--- Algoritmi disponibili ---\n");
				scelta=AlgMenu.Select();			
				if (scelta!=5)             // 5 --> lascia invariato
					QualeAlgoritmo=scelta;  // applica la scelta
				break;
			case 3:	//-------------- CAMBIA AVVERSARIO --------------
				printf("--- Modalita' Avversario ---\n");
				scelta=AvvMenu.Select();			
				if (scelta!=3)             // 3 --> lascia invariato
					QualeAvversario=scelta; // applica la scelta
				break;
         case 4:	//-------------- CAMBIA SCATTAFOTO --------------
				printf("--- Modalita' Scatta Foto ---\n");
            printf("(Attenzione! E' lento a preparare le immagini!)\n");
				scelta=FotoMenu.Select();			
				if (scelta!=9)             // 9 --> lascia invariato
               ScattaFoto=FotoMenu_Mosse(scelta);
           
				break;
			case 5:  //------------------ VISUALIZZA ---------------------
         	if (G==NULL)   // il grafo e' caricato?
				{
					printf("ERRORE! Nessun grafo caricato!\n");
					break;
				}
            VediGrafo();            
            break;
			case 6:  //------------------ ESEGUI ---------------------
				if (G==NULL)   // il grafo e' caricato?
				{
					printf("ERRORE! Nessun grafo caricato!\n");
					break;
				}
            if (ScattaFoto)
            {
               char Buf2[256];
               do
               {
                  printf("Dai un nome ai fotogrammi: ");
                  fgets(Buf2,128,stdin);
                  if (strlen(Buf2)==0) continue;
                  Buf2[strlen(Buf2)-1]='\0';
               } while (strlen(Buf2)==0);
               system ("mkdir Foto >& /dev/null");
               sprintf(Buf,"mkdir Foto/%s >& /dev/null",Buf2);
               system(Buf);
              
               // metto in Buf il percorso dove salvare le foto
               sprintf(Buf,"Foto/%s/",Buf2);
            }
            // verifica l'avversario
				avvmode=true;
				if (QualeAvversario==AVV_AUTO)
					avvmode=false;
					
				// crea l'interfaccia
				TheRI = new RInterface(G,NodoPartenza,avvmode,ScattaFoto,Buf);
           
				// crea il robot appropriato
				switch(QualeAlgoritmo)
				{
					case ALG_DFS:
						TheBot=new DfsBOT(TheRI);break;
					case ALG_GREEDY:
						TheBot=new GreedyBOT(TheRI);break;
					case ALG_PANAITE:
						TheBot=new PanaiteBOT(TheRI,false);break;
  					case ALG_PANAITE_2:
						TheBot=new PanaiteBOT(TheRI,true);break;

				}
				
				// iniza l'esplorazione!
				TheBot->Explore();
				fflush(stdin);
            printf("--> Premi C per continuare\n");
            char Buf[4];
            do
            {
               fgets(Buf,2,stdin);
            } while (Buf[0]!='c' && Buf[0]!='C');
            
				// salva la mappa costruita dal robot
				printf("-> Sto salvando la memoria del bot...\n");
				TheBot->SaveMap("temp2.txt");

				printf("-> Sto controllando se il bot ha esplorato\n");
				printf("   e mappato l'intero grafo...\n");

				// ricarica la mappa del robot
				RobotMap=new Grafo();
				RobotMap->Load("temp2.txt",false);

				// controlla se sono uguali 
            // (cioe' se il robot ha esplorato correttamente)
   			ok=G->IsEqual(RobotMap);
				
				delete RobotMap; // non ci serve piu'

				if (ok)
					printf("OK!!\n");
				else
					printf("ERRORE! I grafi non coincidono!\n");

            if (ScattaFoto)
            {
               // prepara le foto (se ci sono)
               printf("ATTENZIONE! Preparare tutte le fotografie richiede tempo!\n");
               TheRI->SviluppaFoto();
            }

            // distruggi robot e interfaccia
				delete TheBot;
				delete TheRI;
												
				break;
			case 7:  //------------------ USCITA ---------------------
				exit=true;  // condizione d'uscita
            
				if (G)      // se e' presente un grafo
					delete G;// elimina
				G=NULL;
				break;
		}
	}
	
	return 0;
}